#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-03-24
#
# only use full for Cisco WSA appliances
#

from typing import Mapping, List, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    Result,
    SNMPTree,
    contains,
    State,
    render,
)


class CiscoAsyncosBandwidth(NamedTuple):
    bandwidth_now: int
    bandwidth_hour: int
    bandwidth_day: int


# [[['0', '0', '247']]]
def parse_cisco_asyncos_bandwidth(string_table: List[StringTable]) -> CiscoAsyncosBandwidth:
    return CiscoAsyncosBandwidth(
        bandwidth_now=int(string_table[0][0][0]),
        bandwidth_hour=int(string_table[0][0][1]),
        bandwidth_day = int(string_table[0][0][2]),
    )


def discovery_cisco_asyncos_bandwidth(section: CiscoAsyncosBandwidth) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_bandwidth(params, section: CiscoAsyncosBandwidth) -> CheckResult:
    yield from check_levels(
        section.bandwidth_now * 8,
        label='Current bandwidth',
        levels_upper=params.get('upper', None),
        levels_lower=params.get('lower', None),
        metric_name='bandwith_now',
        render_func=render.networkbandwidth,
        # unit='bits/s'
    )
    yield from check_levels(
        section.bandwidth_hour * 8,
        label='last hour',
        # levels_upper=params.get('upper', None),
        # levels_lower=params.get('lower', None),
        # metric_name='bandwith_now',
        render_func=render.networkbandwidth
    )
    yield from check_levels(
        section.bandwidth_day * 8,
        label='last day',
        #levels_upper=params.get('upper', None),
        #levels_lower=params.get('lower', None),
        #metric_name='bandwith_now',
        render_func=render.networkbandwidth
    )

    # yield Result(state=State.OK, summary='last hour: %s, last day: %s' % (section.bandwidth_hour, section.bandwidth_day))


register.snmp_section(
    name='cisco_asyncos_bandwidth',
    parse_function=parse_cisco_asyncos_bandwidth,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.15497.1.2.3.7.4',  # ASYNCOSWEBSECURITYAPPLIANCE-MIB::proxyRecentBandWTotPerf
            oids=[
                '1',  # cacheBwidthTotalNow
                '3',  # cacheBwidthTotal1hrMean
                '5',  # cacheBwidthTotal1dayMean
            ]
        ),
    ],
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_bandwidth',
    service_name='Bandwidth',
    discovery_function=discovery_cisco_asyncos_bandwidth,
    check_function=check_cisco_asyncos_bandwidth,
    check_default_parameters={},
)
