#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-03-24
#
# only use full for Cisco WSA appliances
#

from typing import Mapping, List, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    Result,
    Metric,
    SNMPTree,
    contains,
    State,
)


class CiscoAsyncosCache(NamedTuple):
    cache_hits_now: int
    cache_misses_now: int
    cache_hit_resptime_now: int
    cache_miss_resptime_now: int
    cache_total_resptime: int


# [[['0', '0', '0', '0', '0']]]
def parse_cisco_asyncos_cache(string_table: List[StringTable]) -> CiscoAsyncosCache:
    return CiscoAsyncosCache(
        cache_hits_now=int(string_table[0][0][0]),
        cache_misses_now=int(string_table[0][0][1]),
        cache_hit_resptime_now=int(string_table[0][0][2]),
        cache_miss_resptime_now=int(string_table[0][0][3]),
        cache_total_resptime=int(string_table[0][0][4]),
    )


def discovery_cisco_asyncos_cache(section: CiscoAsyncosCache) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_cache(params, section: CiscoAsyncosCache) -> CheckResult:
    yield Result(state=State.OK,
                 summary='Cache Stats current %s Hits (%sms response time), %s Misses (%sms response time) and %sms total response time' % (
                 section.cache_hits_now, section.cache_hit_resptime_now, section.cache_misses_now,
                 section.cache_miss_resptime_now, section.cache_total_resptime))

    yield Metric(name='cache_hits_now', value=section.cache_hits_now)
    yield Metric(name='cache_misses_now', value=section.cache_misses_now)
    yield Metric(name='cache_hit_resptime_now', value=section.cache_hit_resptime_now)
    yield Metric(name='cache_miss_resptime_now', value=section.cache_miss_resptime_now)
    yield Metric(name='cache_total_resptime', value=section.cache_total_resptime)

register.snmp_section(
    name='cisco_asyncos_cache',
    parse_function=parse_cisco_asyncos_cache,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.15497.1.2.3.7',  # ASYNCOSWEBSECURITYAPPLIANCE-MIB::proxyRecentPerf
            oids=[
                '5.1',  # cacheHitsNow
                '6.1',  # cacheMissesNow
                '7.1',  # cacheHitRespTimeNow
                '8.1',  # cacheMissRespTimeNow
                '9.1',  # cacheTotalRespTimeNow
            ]
        ),
    ],
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_cache',
    service_name='Cache Stats',
    discovery_function=discovery_cisco_asyncos_cache,
    check_function=check_cisco_asyncos_cache,
    check_default_parameters={},
)
