#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-03-24
#
# only use full for Cisco WSA appliances
#
from typing import Mapping, List, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    Result,
    Metric,
    SNMPTree,
    contains,
    State,
)


class CiscoAsyncosConn(NamedTuple):
    cacheClientIdleConns: int
    cacheClientTotalConns: int
    cacheClientMaxConns: int


# [[['0', '5', '66277']]]
def parse_cisco_asyncos_conn(string_table: List[StringTable]) -> CiscoAsyncosConn:
    return CiscoAsyncosConn(
        cacheClientIdleConns=int(string_table[0][0][0]),
        cacheClientTotalConns=int(string_table[0][0][1]),
        cacheClientMaxConns=int(string_table[0][0][2]),
    )


def discovery_cisco_asyncos_conn(section: CiscoAsyncosConn) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_conn(params, section: CiscoAsyncosConn) -> CheckResult:
    yield Result(state=State.OK,
                 summary='%s Active Connections - %s Idle Connections - Maximum Connections was %s' % (
                     section.cacheClientTotalConns, section.cacheClientIdleConns, section.cacheClientMaxConns))

    yield Metric(name='cacheClientIdleConns', value=section.cacheClientIdleConns)
    yield Metric(name='cacheClientTotalConns', value=section.cacheClientTotalConns)
    yield Metric(name='cacheClientMaxConns', value=section.cacheClientMaxConns)


register.snmp_section(
    name='cisco_asyncos_conn',
    parse_function=parse_cisco_asyncos_conn,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.15497.1.2.3.2',  # ASYNCOSWEBSECURITYAPPLIANCE-MIB::proxyClientSidePerf
            oids=[
                '7',  # cacheClientIdleConns
                '8',  # cacheClientTotalConns
                '9',  # cacheClientMaxConns
            ]
        ),
    ],
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_conn',
    service_name='Connection Stats',
    discovery_function=discovery_cisco_asyncos_conn,
    check_function=check_cisco_asyncos_conn,
    check_default_parameters={},
)
