#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-03-08
#
# monitors status Cisco IronPort Appliances (ESA) DNS requests
# Tested with: C380, C370
#
# 2021-03-24: rewrite for CMK 2.0
#
# .1.3.6.1.4.1.15497.1.1.1.15.0 = Gauge32: 0
# .1.3.6.1.4.1.15497.1.1.1.16.0 = Gauge32: 0
#
# ASYNCOS-MAIL-MIB::outstandingDNSRequests.0 = Gauge32: 0
# ASYNCOS-MAIL-MIB::pendingDNSRequests.0 = Gauge32: 0
#
#
from typing import Mapping, List, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    Result,
    Metric,
    SNMPTree,
    contains,
    State,
)


class CiscoAsyncosDns(NamedTuple):
    outstandingDNSRequests: int
    pendingDNSRequests: int


# [[['0', '0']]]
def parse_cisco_asyncos_dns(string_table: List[StringTable]) -> CiscoAsyncosDns:
    return CiscoAsyncosDns(
        outstandingDNSRequests=int(string_table[0][0][0]),
        pendingDNSRequests=int(string_table[0][0][1]),
    )


def discovery_cisco_asyncos_dns(section: CiscoAsyncosDns) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_dns(params, section: CiscoAsyncosDns) -> CheckResult:
    yield Result(state=State.OK,
                 summary='%s outstanding, %s pending' % (
                     section.outstandingDNSRequests, section.pendingDNSRequests))

    yield Metric(name='outstandingDNSRequests', value=section.outstandingDNSRequests)
    yield Metric(name='pendingDNSRequests', value=section.pendingDNSRequests)


register.snmp_section(
    name='cisco_asyncos_dns',
    parse_function=parse_cisco_asyncos_dns,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.15497.1.1.1',  # ASYNCOS-MAIL-MIB
            oids=[
                '15',  # outstandingDNSRequests
                '16',  # pendingDNSRequests
            ]
        ),
    ],
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_dns',
    service_name='DNS requests',
    discovery_function=discovery_cisco_asyncos_dns,
    check_function=check_cisco_asyncos_dns,
    check_default_parameters={},
)
