#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-03-08
#
# monitors status Cisco IronPort Appliances (ESA) oldest message age
# Tested with: C380, C370
#
# 2021-03-24: rewrite for CMK 2.0
#
# .1.3.6.1.4.1.15497.1.1.1.14.0 = Gauge32: 258454
#
# ASYNCOS-MAIL-MIB::oldestMessageAge.0 = Gauge32: 259706
#

from typing import Mapping, List, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    Result,
    Metric,
    SNMPTree,
    contains,
    State,
    render,
)


class CiscoAsyncosMessageage(NamedTuple):
    messageage: int

# [[['259297']]]
def parse_cisco_asyncos_messageage(string_table: List[StringTable]) -> CiscoAsyncosMessageage:
    return CiscoAsyncosMessageage(
        messageage=int(string_table[0][0][0]),
    )


def discovery_cisco_asyncos_messageage(section: CiscoAsyncosMessageage) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_messageage(params, section: CiscoAsyncosMessageage) -> CheckResult:
    yield from check_levels(
        value=section.messageage,
        label='Oldest messages',
        metric_name='message_age',
        levels_upper=params.get('upper', None),
        render_func=render.timespan,
    )


register.snmp_section(
    name='cisco_asyncos_messageage',
    parse_function=parse_cisco_asyncos_messageage,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.15497.1.1.1',  # ASYNCOS-MAIL-MIB
            oids=[
                '14',  # oldestMessageAge
            ]
        ),
    ],
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_messageage',
    service_name='Message age',
    discovery_function=discovery_cisco_asyncos_messageage,
    check_function=check_cisco_asyncos_messageage,
    check_default_parameters={},
)
