#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-02-10
#
#  added wato, fixed inventory function, changed snmp scan function, changed item to 'work queue'
#
# 2021-03-24: rewrite for CMK 2.0
#

from typing import Mapping, List, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    Result,
    Metric,
    SNMPTree,
    contains,
    State,
)


class CiscoAsyncosQueue(NamedTuple):
    state: State
    status_readable: str
    workQueueMessages: int


def get_status_readable(st: str) -> str:
    states = {
        '1': 'enough',
        '2': 'near full',
        '3': 'full',
    }
    return states.get(st, st)


def get_cmk_state(st: str) -> State:
    states = {
        '1': State.OK,
        '2': State.CRIT,
        '3': State.WARN
    }
    return states.get(st, State.CRIT)


# [[['1', '1']]]
def parse_cisco_asyncos_messageage(string_table: List[StringTable]) -> CiscoAsyncosQueue:
    return CiscoAsyncosQueue(
        state=get_cmk_state(string_table[0][0][0]),
        status_readable=get_status_readable(string_table[0][0][0]),
        workQueueMessages=int(string_table[0][0][1])
    )


def discovery_cisco_asyncos_queue(section: CiscoAsyncosQueue) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_queue(params, section: CiscoAsyncosQueue) -> CheckResult:
    yield from check_levels(
        value=section.workQueueMessages,
        label='Messages in work queue',
        levels_upper=params.get('upper', None),
        metric_name='work_queue_size',
    )
    yield Result(state=section.state, notice='Queue space: %s' % section.status_readable)


register.snmp_section(
    name='cisco_asyncos_queue',
    parse_function=parse_cisco_asyncos_messageage,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.15497.1.1.1',  # ASYNCOS-MAIL-MIB
            oids=[
                '5',  # queueAvailabilityStatus
                '11'  # workQueueMessages
            ]
        ),
    ],
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_queue',
    service_name='Queue',
    discovery_function=discovery_cisco_asyncos_queue,
    check_function=check_cisco_asyncos_queue,
    check_default_parameters={'levels': (50, 100)},
    check_ruleset_name='cisco_asyncos_queue',
)
