#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-03-08
#
# monitors status Cisco IronPort Appliances (ESA) system resources
# Tested with: C380, C370
#
# 2021-03-25: rewrite for CMK2.0
#
# .1.3.6.1.4.1.15497.1.1.1.6.0 = INTEGER: 1
#
# ASYNCOS-MAIL-MIB::resourceConservationReason.0 = INTEGER: noResourceConservation(1)
#
#
from typing import Mapping, List, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    Result,
    SNMPTree,
    contains,
    State,
    render,
)


def get_status_readable(st: str) -> str:
    states = {
        '1': 'none',
        '2': 'memory shortage',
        '3': 'queue space shortage',
        '4': 'queue Full',
    }
    return states.get(st, st)


def get_cmk_state(st: str) -> State:
    states = {
        '1': State.OK,
        '2': State.WARN,
        '3': State.WARN,
        '4': State.CRIT,
    }
    return states.get(st, State.CRIT)


class CiscoAsyncosResources(NamedTuple):
    state: State
    status_readable: str


def parse_cisco_asyncos_bandwidth(string_table: List[StringTable]) -> CiscoAsyncosResources:
    return CiscoAsyncosResources(
        state=get_cmk_state(string_table[0][0][0]),
        status_readable=get_status_readable(string_table[0][0][0])
    )


def discovery_cisco_asyncos_resources(section: CiscoAsyncosResources) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_resources(section: CiscoAsyncosResources) -> CheckResult:
    yield Result(state=section.state, notice='Resource conservation %s' % section.status_readable)


register.snmp_section(
    name='cisco_asyncos_resources',
    parse_function=parse_cisco_asyncos_bandwidth,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.15497.1.1.1',  # ASYNCOS-MAIL-MIB
            oids=[
                '6',  # resourceConservationReason
            ]
        ),
    ],
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_resources',
    service_name='Resources',
    discovery_function=discovery_cisco_asyncos_resources,
    check_function=check_cisco_asyncos_resources,
)
