#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-02-10
#
# monitors status Cisco IronPort Appliances (ESA/SMA/WSA) temperature sensors
# Tested with: C380, M380, C370, M670, S370
#
# 2021-03-21: rewrite for CMK 2.0
#
# sample snmpwalk
# .1.3.6.1.4.1.15497.1.1.1.9.1.1.1 = INTEGER: 1
# .1.3.6.1.4.1.15497.1.1.1.9.1.2.1 = INTEGER: 22
# .1.3.6.1.4.1.15497.1.1.1.9.1.3.1 = STRING: Ambient
#
# ASYNCOS-MAIL-MIB::temperatureIndex.1 = INTEGER: 1
# ASYNCOS-MAIL-MIB::degreesCelsius.1 = INTEGER: 22
# ASYNCOS-MAIL-MIB::temperatureName.1 = STRING: Ambient
#

from typing import Mapping, Dict, List, Tuple, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    SNMPTree,
    contains,
)

from cmk.base.plugins.agent_based.utils.temperature import (
    check_temperature,
    TempParamType,
)


#
# [[['22', 'FP_TEMP_SENSOR']]]
#
def parse_cisco_asyncos_temp(string_table: List[StringTable]) -> dict:
    sensors = {}
    for sensor in string_table[0]:
        try:
            value, item = sensor
            item = item.replace('_', ' ')
            sensors.update({item: int(value)})
        except (ValueError, IndexError):
            pass
    return sensors


def discovery_cisco_asyncos_temp(section: Dict) -> DiscoveryResult:
    for key in section.keys():
        yield Service(item=key)


def check_cisco_asyncos_temp(item, params: TempParamType, section) -> CheckResult:
    try:
        yield from check_temperature(
            section[item],
            params=params,
            unique_name='check_cisco_asyncos_temp.%s' % item,
        )
    except KeyError:
        pass


register.snmp_section(
    name='cisco_asyncos_temp',
    parse_function=parse_cisco_asyncos_temp,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.15497.1.1.1.9.1',  # ASYNCOS-MAIL-MIB::temperatureEntry
            oids=[
                '2',  # degreesCelsius
                '3',  # temperatureName
            ]
        ),
    ],
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_temp',
    service_name='Temperature %s',
    discovery_function=discovery_cisco_asyncos_temp,
    check_function=check_cisco_asyncos_temp,
    check_default_parameters={},
    check_ruleset_name='temperature'
)
